@@ -3,14 +3,10 @@
 from __future__ import with_statement
 
 import unittest
-import mailbox
 
-from email import encoders
+from flask import Flask
+from flask_mail import Mail, Message, BadHeaderError
 
-from flask import Flask, g
-from flaskext.mail import Mail, Message, BadHeaderError, Attachment
-
-from nose.tools import assert_equal
 
 class TestCase(unittest.TestCase):
 
@@ -21,8 +17,8 @@ class TestCase(unittest.TestCase):
 
         self.app = Flask(__name__)
         self.app.config.from_object(self)
-        
-        assert self.app.testing
+
+        self.assertTrue(self.app.testing)
 
         self.mail = Mail(self.app)
 
@@ -41,40 +37,48 @@ class TestMessage(TestCase):
         msg = Message(subject="subject",
                       recipients=["to@example.com"])
 
-
-        assert msg.sender == "support@mysite.com"
-        assert msg.recipients == ["to@example.com"]
+        self.assertEqual(msg.sender, "support@mysite.com")
+        self.assertEqual(msg.recipients, ["to@example.com"])
 
     def test_recipients_properly_initialized(self):
 
         msg = Message(subject="subject")
-
-        assert msg.recipients == []
+        self.assertEqual(msg.recipients, [])
 
         msg2 = Message(subject="subject")
         msg2.add_recipient("somebody@here.com")
+        self.assertEqual(len(msg2.recipients), 1)
 
-        assert len(msg.recipients) == 0
-
-        msg3 = Message(subject="subject")
-        msg3.add_recipient("somebody@here.com")
-
-        assert len(msg.recipients) == 0
+    def test_sendto_properly_set(self):
+        msg = Message(subject="subject", recipients=["somebody@here.com"],
+                       cc=["cc@example.com"], bcc=["bcc@example.com"])
+        self.assertEqual(len(msg.send_to), 3)
+        msg.add_recipient("cc@example.com")
+        self.assertEqual(len(msg.send_to), 3)
 
     def test_add_recipient(self):
 
         msg = Message("testing")
         msg.add_recipient("to@example.com")
 
-        assert msg.recipients == ["to@example.com"]
+        self.assertEqual(msg.recipients, ["to@example.com"])
 
-    
     def test_sender_as_tuple(self):
-
         msg = Message(subject="testing",
                       sender=("tester", "tester@example.com"))
+        self.assertEqual('tester <tester@example.com>', msg.sender)
+
+    def test_reply_to(self):
+
+        msg = Message(subject="testing",
+                      recipients=["to@example.com"],
+                      sender="spammer <spammer@example.com>",
+                      reply_to="somebody <somebody@example.com>",
+                      body="testing")
+
+        response = msg.as_string()
+        self.assertIn("Reply-To: somebody <somebody@example.com>", str(response))
 
-    
     def test_send_without_sender(self):
 
         del self.app.config['DEFAULT_MAIL_SENDER']
@@ -107,7 +111,7 @@ class TestMessage(TestCase):
     def test_normal_send(self):
         """
         This will not actually send a message unless the mail server
-        is set up. The error will be logged but test should still 
+        is set up. The error will be logged but test should still
         pass.
         """
 
@@ -121,9 +125,9 @@ class TestMessage(TestCase):
                           body="testing")
 
             self.mail.send(msg)
-            
-            assert len(outbox) == 1
-        
+
+            self.assertEqual(len(outbox), 1)
+
         self.app.config['TESTING'] = True
 
     def test_bcc(self):
@@ -133,8 +137,8 @@ class TestMessage(TestCase):
                       body="testing",
                       bcc=["tosomeoneelse@example.com"])
 
-        response = msg.get_response()
-        assert "Bcc: tosomeoneelse@example.com" in str(response)
+        response = msg.as_string()
+        self.assertIn("Bcc: tosomeoneelse@example.com", str(response))
 
     def test_cc(self):
 
@@ -143,26 +147,24 @@ class TestMessage(TestCase):
                       body="testing",
                       cc=["tosomeoneelse@example.com"])
 
-        response = msg.get_response()
-        assert "Cc: tosomeoneelse@example.com" in str(response)
+        response = msg.as_string()
+        self.assertIn("Cc: tosomeoneelse@example.com", str(response))
 
     def test_attach(self):
 
         msg = Message(subject="testing",
                       recipients=["to@example.com"],
                       body="testing")
-        
-        msg.attach(data="this is a test", 
+
+        msg.attach(data="this is a test",
                    content_type="text/plain")
-        
 
         a = msg.attachments[0]
-        
-        assert a.filename is None
-        assert a.disposition == 'attachment'
-        assert a.content_type == "text/plain"
-        assert a.data == "this is a test"
- 
+
+        self.assertIsNone(a.filename)
+        self.assertEqual(a.disposition, 'attachment')
+        self.assertEqual(a.content_type, "text/plain")
+        self.assertEqual(a.data, "this is a test")
 
     def test_bad_header_subject(self):
 
@@ -182,6 +184,16 @@ class TestMessage(TestCase):
 
         self.assertRaises(BadHeaderError, self.mail.send, msg)
 
+    def test_bad_header_reply_to(self):
+
+        msg = Message(subject="testing",
+                      sender="from@example.com",
+                      reply_to="evil@example.com\n\r",
+                      recipients=["to@example.com"],
+                      body="testing")
+
+        self.assertRaises(BadHeaderError, self.mail.send, msg)
+
     def test_bad_header_recipient(self):
 
         msg = Message(subject="testing",
@@ -193,6 +205,37 @@ class TestMessage(TestCase):
 
         self.assertRaises(BadHeaderError, self.mail.send, msg)
 
+    def test_plain_message(self):
+
+        plain_text = "Hello Joe,\nHow are you?"
+
+        msg = Message(subject="subject",
+                      recipients=["to@example.com"],
+                      body=plain_text)
+
+        self.assertEqual(plain_text, msg.body)
+        self.assertIn('Content-Type: text/plain', msg.as_string())
+
+    def test_plain_message_with_attachments(self):
+        msg = Message(subject="subject",
+                      recipients=["to@example.com"],
+                      body="hello")
+
+        msg.attach(data="this is a test",
+                   content_type="text/plain")
+
+        self.assertIn('Content-Type: multipart/mixed', msg.as_string())
+
+    def test_html_message(self):
+        html_text = "<p>Hello World</p>"
+
+        msg = Message(subject="subject",
+                      recipients=["to@example.com"],
+                      html=html_text)
+
+        self.assertEqual(html_text, msg.html)
+        self.assertIn('Content-Type: multipart/alternative', msg.as_string())
+
 
 class TestMail(TestCase):
 
@@ -205,7 +248,7 @@ class TestMail(TestCase):
 
             self.mail.send(msg)
 
-            assert len(outbox) == 1 
+            self.assertEqual(len(outbox), 1)
 
     def test_send_message(self):
 
@@ -214,13 +257,13 @@ class TestMail(TestCase):
                                    recipients=["tester@example.com"],
                                    body="test")
 
-            assert len(outbox) == 1
+            self.assertEqual(len(outbox), 1)
 
             msg = outbox[0]
 
-            assert msg.subject == "testing"
-            assert msg.recipients == ["tester@example.com"]
-            assert msg.body == "test"
+            self.assertEqual(msg.subject, "testing")
+            self.assertEqual(msg.recipients, ["tester@example.com"])
+            self.assertEqual(msg.body, "test")
 
 
 class TestConnection(TestCase):
@@ -233,7 +276,7 @@ class TestConnection(TestCase):
                                   recipients=["to@example.com"],
                                   body="testing")
 
-            assert len(outbox) == 1
+            self.assertEqual(len(outbox), 1)
 
     def test_send_single(self):
 
@@ -245,11 +288,9 @@ class TestConnection(TestCase):
 
                 conn.send(msg)
 
-            assert len(outbox) == 1
+            self.assertEqual(len(outbox), 1)
 
     def test_send_many(self):
-        
-        messages = []
 
         with self.mail.record_messages() as outbox:
             with self.mail.connect() as conn:
@@ -257,14 +298,12 @@ class TestConnection(TestCase):
                     msg = Message(subject="testing",
                                   recipients=["to@example.com"],
                                   body="testing")
-        
+
                     conn.send(msg)
 
-            assert len(outbox) == 100
+            self.assertEqual(len(outbox), 100)
 
     def test_max_emails(self):
-        
-        messages = []
 
         with self.mail.record_messages() as outbox:
             with self.mail.connect(max_emails=10) as conn:
@@ -272,12 +311,10 @@ class TestConnection(TestCase):
                     msg = Message(subject="testing",
                                   recipients=["to@example.com"],
                                   body="testing")
-        
+
                     conn.send(msg)
 
-                    print conn.num_emails
                     if i % 10 == 0:
-                        assert conn.num_emails == 1
-
-            assert len(outbox) == 100
+                        self.assertEqual(conn.num_emails, 1)
 
+            self.assertEqual(len(outbox), 100)
